<?php
namespace Teknisa\CMP\Modules\Cadastros\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\CMP\Modules\Cadastros\Services\cmp00100ProdutoPorFornecedorService;

class cmp00100ProdutoPorFornecedorController extends \Zeedhi\Framework\Controller\Report
{
    protected $connection;
    protected $newCode;
    protected $sheetImporter;
    protected $birtStrategy;
    
    public function __construct(ReportService $reportService, cmp00100ProdutoPorFornecedorService $service, $connection, $newCode, $environment, $sheetImporter, $birtStrategy, $entityManager)
    {
        parent::__construct($reportService);
        $this->connection    = $connection;
        $this->service       = $service;
        $this->newCode       = $newCode;
        $this->environment   = $environment;
        $this->sheetImporter = $sheetImporter;
        $this->birtStrategy      = $birtStrategy;
        $this->entityManager     = $entityManager;
    }
    
    public function generateCode($table)
    {
        return $this->newCode->getCodeByTableName($table, $this->environment->getNrOrg());
    }
    
    public function concatArray($array)
    {
        return join('_', $array);
    }
    
    public function completaProd($campo)
    {
        return str_pad($campo, 20, 'Z', STR_PAD_RIGHT);
    }
    
    public function saveProduto(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        $searchProduto = $this->service->findProduto($row);
        $validateProduto = $this->service->validateProduto($row);

        if (empty($searchProduto) && empty($validateProduto)) {
            $this->service->deleteQProdForn($row);
            $this->service->deleteProdForn($row);
            $this->service->insertProduto($row);
            $this->service->procInsertProduto($row);
        } else {
            $erro = new Error("Operação bloqueada! Produto ou Grupo já cadastrado", 500);
            $response->setError($erro);
        }
    }
    
    public function saveProdutoNvlCompleto(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $this->service->updateProdFornComp($row);
    }
    
    public function saveFormaPagamento(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $this->service->updateProdForn($row[1]);
        if ($row[1]['IDTPFORPGTFO'] == 'V') {
            $this->service->deletePrazos($row[1]);

            foreach ($row[0] as $data) {
                $this->service->insertParcelas($data, $row[1]);
            }
        } else {
            $this->service->deletePrazos($row[1]);
        }
    }
    
    public function geraLista($CDFORNECEDDEST, $nrorg)
    {
        $nrlista = $this->generateCode('LISTA');
        if ($CDFORNECEDDEST != 'T') {
            if (!empty($CDFORNECEDDEST)) {
                foreach ($CDFORNECEDDEST as $cdforneced) {
                    $this->service->insertLista($nrlista, $cdforneced, $nrorg);
                } 
            }
        }
        return $nrlista;
    }
    
    public function exportProduto(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        if ($row['EXPROD'] != null) {
            $codLista = $this->geraLista($row['CDFORNECEDDEST'], $row['NRORG']);       
            
            if ($row['EXPROD'] == 'T') {
                $this->service->deleteProdFornTodos($codLista);
            } else {
                $this->service->deleteExpProdForn($codLista, $row['CDARVPRODINI'], $row['CDARVPRODFIN']);
            }
            
            $this->service->deleteLista($codLista);
        }
        
        if ($row['ARRENDEMP'] == 'S') {
            $ARRENDEMP = 'E';
        } else {
            $ARRENDEMP = '';
        }
        
        if (is_array($row['CDFORNECEDDEST'])) {
            $fornecedorDest = $this->concatArray($row['CDFORNECEDDEST']);
        } else {
            $fornecedorDest = $row['CDFORNECEDDEST'];
        }
        
        $prod = $this->service->getCodProd($row, $fornecedorDest);

        if (!empty($prod)) {
            foreach ($prod as $value) {
                $this->service->insertProdForn($value);
                $varifyProdForn = $this->service->verifyDupProdForn($value);
                
                if (!empty($varifyProdForn)) {
                    $this->service->updateVerifiedProdForn($value, $row['CDFORNECEDORIG']);
                    $this->service->insertPrazoPgt($value, $row['CDFORNECEDORIG']);
                    
                    $value['ARRED'] = $ARRENDEMP;
                    $value['CDFORNORIG'] = $row['CDFORNECEDORIG'];
                    $value['CDMARCA'] = '';
                    
                    $this->service->procInsertProduto($value);
                }
            } 
        }
    }
    
    public function cadProdAuto(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        $row['CDARVPRODFIN'] = self::completaProd($row['CDARVPRODFIN']);
        
        foreach ($row['CDFORNECED'] as $fornecedor) {
            $queryProdutosCad = $this->service->getProdCad($row, $fornecedor);

            foreach ($queryProdutosCad as $prod) {
                $prod['CDFORNECED'] = $fornecedor;
                $prod['CDMARCA']    = null;
                $prod['ARRED']      = 'P';
                $prod['CDFORNORIG'] = null;
                
                $this->service->insertProdForn($prod);
                $this->service->procInsertProduto($prod);
            } 
        }
    }
    
    public function altProdAuto(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        if (is_array($row['CDPRODUTO'])) {
            $row['CDPRODUTO'] = is_array($row['CDPRODUTO']) && !empty($row['CDPRODUTO']) ? self::concatArray($row['CDPRODUTO']) : 'T';
        }
        
        $row['CDARVPRODFIN'] = self::completaProd($row['CDARVPRODFIN']);
        $queryProdutosCad = $this->service->updateAlteraAuto($row);
        
        $produtoCadastradoFornecedor = $this->service->verifyProdForn($row);
        $prodNaoCadastradoForn = array();
        $prodNaoCadastradoForn = false;
        if (!$produtoCadastradoFornecedor[0]['COUNT(*)'] > 0){
            $prodNaoCadastradoForn = true;
                $response->addDataSet(new DataSet("prodNaoCadastradoForn", array("prodCadastrado" => $prodNaoCadastradoForn)));
            }
        $response->addDataSet(new DataSet("prodNaoCadastradoForn", array("prodCadastrado" => $prodNaoCadastradoForn)));
    }
    
    public function importSheet(DTO\Request $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();

            $sheetPath = $this->sheetImporter->createTemporaryFile($row["sheetImport"]);
            $worksheet = $this->sheetImporter->loadSheet($sheetPath);
            $sheet = $this->sheetImporter->parseSheet($worksheet);
            
            $this->sheetImporter->validateSheetColumns($sheet);
            $this->sheetImporter->validateSheetRows($sheet, $response);

            $prod = $this->sheetImporter->validateProdForn($row, $sheet, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        } finally {
            if (isset($sheetPath)) {
                $this->sheetImporter->removeTemporaryFile($sheetPath);
            }
        }
    }
    
    public function deleteProduto(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        if (!empty($row)) {
            foreach ($row as $value) {
                $this->service->deleteQProdForn($value);
                $this->service->deleteProdForn($value);
            }
        }
    }
    public function deleteFormaPagamento(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $this->service->deleteFormPag($row);
    }
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) 
    {
        $row = $request->getRow();
        $row['P_NRORG'] = $this->environment->getNrOrg();
        
    
        if (is_array($row['FORNECEDOR'])) {
            $row['FORNECEDOR'] = $this->concatArray($row['FORNECEDOR']);
        }
           
        
        if($row['FORNECEDOR'] == ''){
            $row['FORNECEDOR'] = 'T';
        }
         
        if($row['FORNECEDOR'] != 'T' && $row['FORNECEDOR'] != null){
            $nrlista = $this->criarListaValores($row['FORNECEDOR']);   
            $row['FORNECEDOR'] = "_NRLISTA=".$nrlista;
        }
        
        
        $row['CDARVPRODFIN'] = self::completaProd($row['CDARVPRODFIN']);
        
        $type = isset($row['QR2TYPE']) ? $row['QR2TYPE'] : 'PDF';
        $this->birtStrategy->setReportFormat($type);
        
        $this->stNomeRel = $this->getReportName($row);
        $this->reportMapping[$this->stNomeRel] =
            array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                "FORNECEDOR"   => "L_CDFORNECED",
                "CDARVPRODINI" => "P_CDINI",
                "CDARVPRODFIN" => "P_CDFIM",
                "P_NRORG" => "P_NRORG"
            ),
        );
    }
    
    //cria lista de valores recebendo string como parametro
    public function criarListaValores($valores){
       
       
            $this->entityManager->beginTransaction();
            $lista = explode('_',$valores);
             
            $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
            try{
                foreach($lista as $key){
                    $this->insertLista($nrlista, $key);
                }
                $this->entityManager->commit();
                return $nrlista;
            } catch(\Exception $e){
                $this->entityManager->rollback();
                throw $e;
            }
        
    }
    //insere lista no banco
    public function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista
        );
        
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    
}
