<?php
namespace Teknisa\CMP\Modules\Fornecimento\Modules\AutorizacaoFornecimento\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\CMP\Util\CmpUtilities;
use Teknisa\CMP\Util\Attachment;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\Libs\Util\DateUtil;

class Cmp03900LancamentoAutorizacaoDeFornecimentoControllerReport extends \Zeedhi\Framework\Controller\Report {
    protected $reportService;
    protected $qr2Strategy;
    protected $connection;
    protected $service;
    protected $cmpUtilities;
    protected $stNomeRel;
    protected $idsisdos;
    protected $logEnvioAF;
    protected $cookieController;
    
    protected $globalReports = [];

    public function __construct($reportService, $qr2Strategy, $connection, $service, $birtStrategy, CmpUtilities $cmpUtilities, $cookieController) {
        $this->reportService = $reportService;
        $this->qr2Strategy   = $qr2Strategy;
        $this->connection    = $connection;
        $this->service       = $service;
        $this->birtStrategy  = $birtStrategy;
        $this->cmpUtilities  = $cmpUtilities;
        $this->cookieUtil    = $cookieController;
        $this->logEnvioAF    = array('REPORT' => array(), 'SUCCESS' => array(), 'ERROR' => array());
    }
    
    protected $reportMapping = array(
        'padraoEnvioAF' => array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
                array (
                    'CDFILIAL'       => 'P_CDFILIAL',
                    'NRAUTOFORN'     => 'P_NRAUTOFORN',
                    'CDFILLANC'      => 'P_CDFILLANC',
                    'IMPRIMEOBS'     => 'P_IDEXIBEOBSFIL',
                    'IMPRIMEOBSPROD' => 'P_IMPRIMEOBSPROD',
                    'NRORG'          => 'NRORG',
                ),
            ),
    );
    
    public function geraRelatorio(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
            $this->stNomeRel = $this->getReportName($row);
            
            if (!array_key_exists($this->stNomeRel, $this->reportMapping)) {
                $this->reportMapping[$this->stNomeRel] = array (
                    'strategy' => $row['STRATEGY'],
                    'parameterMapping' => array (
                        'CDFILIAL'       => 'P_CDFILIAL',
                        'NRAUTOFORN'     => 'P_NRAUTOFORN',
                        'CDFILLANC'      => 'P_CDFILLANC',
                        'PROXMES'        => 'P_PROXMES',
                        'IMPRIMEOBS'     => 'P_IMPRIMEOBS',
                        'IMPRIMEOBSPROD' => 'P_IMPRIMEOBSPROD',
                        'NRMENSAUTFOR'   => 'L_NRMENSAUTFOR',
                        'NRORG'          => 'P_NRORG',
                    )
                );
            }
            
            $reportURL = parent::processReport($request);
            $response->addMessage(new DTO\Response\Message($reportURL));
        } catch (\Exception $e) {
            $response->setCriticalError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function verificaParametrosEnvioRelatorioExtra($row) {
        return $row['IDAFFORALIM'] == 'N' && $row['IDSITUATUAAF'] == null && $row['IDBAIXAAF'] == null && 
            ($row['IDAFOPERLOG'] == null || $row['IDAFOPERLOG'] == '1') && 
            (($row['IDAFPRODWEB'] == 'N' || $row['IDAFPRODWEB'] == null) || 
            ($row['IDBAIXAAF'] == '2' || $row['IDBAIXAAF'] == '3'));
    }
    
    public function afterProcessReport(DTO\Request\Row $request, DTO\Response $response, $report) {
        $row = $request->getRow();
        
        $this->globalReports[] = $report;
    }
    
    /**********************************Envio de Email**********************************/
    
    public function sendEmailAFPosicionada(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $dadosEmail = $row['DADOS_EMAIL'];
        $currentRow = $row['CURRENT_ROW'];
    
        $row['__report_name'] = explode('.', $dadosEmail['REPORT'])[0];
        
        $isEmailSend = false;
        $localResponse = new DTO\Response();
        
        try {
            $messages = [];
            
            $dadosForn = $this->service->retornaDadosEmailFornecedor($currentRow['CDFORNECED']);
            
            if($dadosForn) {
                if($dadosForn['IDTPENVAFFOR'] !== '1') {
                    $this->logEnvioAF['ERROR'][] = array("NMFANTFORN" => $dadosForn['NMFANTFORN'], "MESSAGE" => "MAIL_NOT_SENDED_IDTPENVAFFOR");
                } else {
                    $enderecos = explode(";", $dadosForn['DSEMAILCOT']);
                    
                    if(empty($enderecos[0])) {
                        $this->logEnvioAF['ERROR'][] = array("NMFANTFORN" => $dadosForn['NMFANTFORN'], "MESSAGE" => "EMAIL_N_REGISTERED");
                    } else {
                        if($dadosForn['IDTPENVAFNORMAL'] === 'E') {
                            if(isset($dadosEmail["REPORT"])) {
                                $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
                                $this->stNomeRel = $this->getReportName($row);
                                
                                if (!array_key_exists($this->stNomeRel, $this->reportMapping)) {
                                    $this->reportMapping[$this->stNomeRel] = array (
                                        'strategy' => $row['STRATEGY'],
                                        'parameterMapping' => array (
                                            'CDFILIAL'       => 'P_CDFILIAL',
                                            'NRAUTOFORN'     => 'P_NRAUTOFORN',
                                            'CDFILLANC'      => 'P_CDFILLANC',
                                            'PROXMES'        => 'P_PROXMES',
                                            'IMPRIMEOBS'     => 'P_IMPRIMEOBS',
                                            'IMPRIMEOBSPROD' => 'P_IMPRIMEOBSPROD',
                                            'NRMENSAUTFOR'   => 'L_NRMENSAUTFOR',
                                            'NRORG'          => 'P_NRORG',
                                        )
                                    );
                                }
                                
                                $this->globalReports[] = parent::processReport($request);
                            } else {
                                $currentRow['reportLabel']    = 'padraoEnvioAF';
                                $currentRow['__report_name']  = 'padraoEnvioAF';
                                $currentRow['CDFILIAL']       = $currentRow['CDFILDEST'];
                                $currentRow['IMPRIMEOBS']     = $dadosEmail['obsUnidade'];
                                $currentRow['IMPRIMEOBSPROD'] = $dadosEmail['obsProduto'];
            
                                $this->birtStrategy->setReportFormat(strtolower($dadosEmail['EXTENSAO']));
                                parent::createRemoteReport(new DTO\Request\Row(new DTO\Row($currentRow), "", "", ""), $localResponse);
                            }
                        }
                        $isEmailSend = self::sendEmail($currentRow, $dadosEmail, $dadosForn, $enderecos);
                    }
                }
            }
            
            if($isEmailSend) {
                $this->logEnvioAF['SUCCESS'][] = array("NMFANTFORN" => $dadosForn['NMFANTFORN'], "NRAUTOFORN" => $currentRow['NRAUTOFORN'], "MESSAGE" => "EMAIL_SEND_SUCCESS");
            } else {
                $this->logEnvioAF['ERROR'][] = array("NMFANTFORN" => $dadosForn['NMFANTFORN'], "NRAUTOFORN" => $currentRow['NRAUTOFORN'], "MESSAGE" => "EMAIL_SEND_FAILED");
            }
            
            $this->globalReports = [];
            
            $response->addMessage(new Message($this->logEnvioAF, 'W'));
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    private function tratarMensagemCorpoEmail($row, $dadosEmail, $reportName) {
        $dataset = $this->service->retornaLinkEcommerce($row);
        $encontrar  = array("<i>__NMFANTFORN__</i>", "<i>__PARAMPORTAL__</i>", "<i>__NRSEQAPURAF__</i>");
        $substituir = array($row['NMFANTFORN'], $dataset['DSLINKECOMMERCE'], $row['NRSEQAPURAF']);
        
        $message    = str_replace($encontrar, $substituir, $dadosEmail['MENSAGEM']);
        
        $attachments = [];
        $attOnBody = '';
        $clearAttachments = true;
        
        if($row['IDTPENVAFNORMAL'] === 'E') {
            foreach ($this->globalReports as $name => $report) {
                $fileName = ($reportName ? $reportName : $row["NRAUTOFORN"]) . '.' . strtolower($dadosEmail["EXTENSAO"]);
                
                $attachment = new Attachment($report, $fileName);
                $attachment->setIsStringAttachment(true);
                
                $attachments[] = $attachment;
                
                $attOnBody.= "<a href= '".str_replace("PDF", "pdf", $report)."'>" . $dadosEmail["DSLINK"] . "</a><br/><br/>";
            }
            
            $message = $attOnBody.$message;
        }
        
        $footer = "<div style=\"font-size: 10px; margin-top: 20px; word-wrap: break-word;\">" . $dadosEmail["MSGCOMPRAS"] . "<br/>" . $dadosEmail["MSGDATESENT"] . DateUtil::getDataAtual() . "</div>";
        
        $bodyMessage = "<table>
                            <tr>
                                <td class='description'>
                                    <br/>" . $message .
                                "</td>
                            </tr>
                            <tr class='footer'>
                                <td>
                                    <br> <p>" . $footer . "</p>
                                </td>
                            </tr>
                        </table>";
        
        return [$bodyMessage, $attachments, $clearAttachments];
    }
    
    private function sendEmail($row, $dadosEmail, $dadosForn, $enderecos, $reportName = null) {
        $this->logEnvioAF['REPORT'][] = $this->globalReports;
        $this->cookieUtil->setCookie($row, $dadosEmail);
        
        $result = self::tratarMensagemCorpoEmail($row, $dadosEmail, $reportName);
        
        $message          = $result[0];
        $attachments      = $result[1];
        $clearAttachments = $result[2];
        
        $messageSendMail = $this->cmpUtilities->messageSendMail(null, $dadosEmail["ASSUNTO"], $message, 'TECFOOD');
        $enderecosCopia = !empty($dadosForn["DSMAILCOTCOP"]) ? explode(";", $dadosForn["DSMAILCOTCOP"]) : false;
        
        if($dadosEmail['SENDOPERATOR'] === 'S') {
            $enderecosCopia[] = $this->service->getCurrentEmail();
        }
        
        if(empty($enderecos[0])) {
            $this->logEnvioAF['ERROR'][] = array("NMFANTFORN" => $row['NMFANTFORN'], "MESSAGE" => "EMAIL_N_REGISTERED");
        } else {
            try{
                return $this->cmpUtilities->enviaEmail('', $dadosEmail["ASSUNTO"], $enderecos, $messageSendMail, $enderecos, $enderecosCopia, $attachments, $clearAttachments);
            } catch (\Exception $e) {
                return false;
            }
        }
    }
    
    private function groupedAFs($dataset, $dadosEmail, $filterRow) {
        if(!empty($dataset)) {
            $isEmailSend = false;
            $localResponse = new DTO\Response();
            
            $fornecedores = array_unique(array_map(function($row) {
                return $row['CDFORNECED'];
            }, $dataset));
                
            foreach ($fornecedores as $value) {
                $dadosForn = $this->service->retornaDadosEmailFornecedor($value);
                
                if($dadosForn['IDTPENVAFFOR'] !== '1') {
                    $this->logEnvioAF['ERROR'][] = array("NMFANTFORN" => $dadosForn['NMFANTFORN'], "MESSAGE" => "MAIL_NOT_SENDED_IDTPENVAFFOR");
                } else {
                    $enderecos = explode(";", $dadosForn['DSEMAILCOT']);
                        
                    if(empty($enderecos[0])) {
                        $this->logEnvioAF['ERROR'][] = array("NMFANTFORN" => $dadosForn['NMFANTFORN'], "MESSAGE" => "EMAIL_N_REGISTERED");
                    } else {
                        if($dadosForn['IDTPENVAFNORMAL'] === 'E') {
                            $params = array_filter($dataset, function($row) use($value) {
                                return $row['CDFORNECED'] == $value;
                            });
                            
                            $cdfildest = join("_", array_unique(array_map(function($row) {
                                return $row['CDFILDEST'];
                            }, $params)));
                                
                            $cdcomprador = join("_", array_unique(array_map(function($row) {
                                return $row['CDOPERADOR'];
                            }, $params)));
                            
                            $cdfilcentcmp = join("_", array_unique(array_map(function($row) {
                                return $row['CDFILCENTCMP'];
                            }, $params)));
                                
                            $cdfillanc = array_unique(array_map(function($row) {
                                return $row['CDFILLANC'];
                            }, $params));
                                
                            foreach($cdfillanc as $cdfilial) {
                                $row = array_pop($params);
                                $row['CDFILDEST'] = $cdfildest;
                                $row['CDFILLANC'] = $cdfilial;
                                $row['CDCOMPRADOR'] = $cdcomprador;
                                $row['NRSEQAPURAF'] = null;
                                $row['IMPRIMEOBSPROD'] = $dadosEmail['obsProduto'];
                                $row['IMPRIMEOBS'] = $dadosEmail['obsUnidade'];
                                $row['DTAFINI'] = $filterRow['DT_AF']['start'];
                                $row['DTAFFIN'] = $filterRow['DT_AF']['end'];
                                $row['DTENTINI'] = $filterRow['DT_ENTREGA']['start'];
                                $row['DTENTFIN'] = $filterRow['DT_ENTREGA']['end'];
                                $row['reportLabel'] = 'padraoEnvioAF';
                                $row['__report_name'] = 'padraoEnvioAF';
                                
                                $this->reportMapping['padraoEnvioAF'] = array (
                                    'strategy' => 'BIRT',
                                    'parameterMapping' => array (
                                        'NRSEQAPURAF'    => 'P_NRSEQAPURAF',
                                        'CDFILLANC'      => 'P_CDFILLANC',
                                        'IMPRIMEOBSPROD' => 'P_IMPRIMEOBSPROD',
                                        'IMPRIMEOBS'     => 'P_IDEXIBEOBSFIL',
                                        'CDFORNECED'     => 'P_CDFORNECED',
                                        'DTENTINI'       => 'P_DTENTREGAINI',
                                        'DTENTFIN'       => 'P_DTENTREGAFIN',
                                        'CDFILDEST'      => 'P_CDFILDEST',
                                        'CDCOMPRADOR'    => 'P_CDOPERADOR',
                                        'DTAFINI'        => 'P_DTAUTOFORNINI',
                                        'DTAFFIN'        => 'P_DTAUTOFORNFIN',
                                        "NRORG"          => 'NRORG'
                                    )
                                );
                                
                                $this->birtStrategy->setReportFormat(strtolower($dadosEmail['EXTENSAO']));
                                
                                parent::createRemoteReport(new DTO\Request\Row(new DTO\Row($row), "", "", ""), $localResponse);
                            }
                        }
                        
                        $dadosForn['CDFILCENTCMP'] = $cdfilcentcmp;
                        $dadosForn['NRSEQAPURAF'] = null;

                        $isEmailSend = self::sendEmail($dadosForn, $dadosEmail, $dadosForn, $enderecos, $cdfilial.$value);
                        
                        if($isEmailSend) {
                            $this->logEnvioAF['SUCCESS'][] = array("NMFANTFORN" => $row['NMFANTFORN'], "NRAUTOFORN" => $row['NRAUTOFORN'], "MESSAGE" => "EMAIL_SEND_SUCCESS");
                        } else {
                            $this->logEnvioAF['ERROR'][] = array("NMFANTFORN" => $row['NMFANTFORN'], "NRAUTOFORN" => $row['NRAUTOFORN'], "MESSAGE" => "EMAIL_SEND_FAILED");//'EMAIL_SEND_FAILED';
                        }
                            
                        $this->globalReports = [];
                    }
                }
            }
        }
    }
    
    private function selectedAFs($checkedRows, $dadosEmail, $filterRow) {
        if(!empty($checkedRows)) {
            $localResponse = new DTO\Response();
            $isEmailSend = false;
            
            foreach ($checkedRows as $value){
                $dadosForn = $this->service->retornaDadosEmailFornecedor($value['CDFORNECED']);
                
                if($dadosForn['IDTPENVAFFOR'] !== '1') {
                    $this->logEnvioAF['ERROR'][] = array("NMFANTFORN" => $dadosForn['NMFANTFORN'], "MESSAGE" => "MAIL_NOT_SENDED_IDTPENVAFFOR");
                } else {
                    $enderecos = explode(";", $dadosForn['DSEMAILCOT']);
                        
                    if(empty($enderecos[0])) {
                        $this->logEnvioAF['ERROR'][] = array("NMFANTFORN" => $dadosForn['NMFANTFORN'], "MESSAGE" => "EMAIL_N_REGISTERED");
                    } else {
                        if($dadosForn['IDTPENVAFNORMAL'] === 'E') {
                            $value['CDFILIAL'] = $value['CDFILDEST'];
                            $value['IMPRIMEOBSPROD'] = $dadosEmail['obsProduto'];
                            $value['IMPRIMEOBS'] = $dadosEmail['obsUnidade'];
                            $value['reportLabel'] = 'padraoEnvioAF';
                            $value['__report_name'] = 'padraoEnvioAF';
                
                            $this->birtStrategy->setReportFormat(strtolower($dadosEmail['EXTENSAO']));
                            parent::createRemoteReport(new DTO\Request\Row(new DTO\Row($value), "", "", ""), $localResponse);
                        }
                    }
                    
                    $value['NRSEQAPURAF'] = null;
                    
                    $isEmailSend = self::sendEmail($value, $dadosEmail, $dadosForn, $enderecos);
                    
                    if($isEmailSend) {
                        $this->logEnvioAF['SUCCESS'][] = array("NMFANTFORN" => $value['NMFANTFORN'], "NRAUTOFORN" => $value['NRAUTOFORN'], "MESSAGE" => "EMAIL_SEND_SUCCESS");
                    } else {
                        $this->logEnvioAF['ERROR'][] = array("NMFANTFORN" => $value['NMFANTFORN'], "NRAUTOFORN" => $value['NRAUTOFORN'], "MESSAGE" => "EMAIL_SEND_FAILED");
                    }
                    
                    $this->globalReports = [];
                }
            }
        }
            
    }
    
    public function sendEmailAFSelecionada(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        try {
            $dadosEmail = $row['DADOS_EMAIL'];
            $dataEnvio = $row['DATASET'];
            $checkedRows = $row['CHECKED_ROWS'];
            $filterRow = $row['FILTER_ROW'];
            
            if($filterRow['IDAGRUPAF']) {
                self::groupedAFs($dataEnvio, $dadosEmail, $filterRow);
            } else {
                self::selectedAFs($checkedRows, $dadosEmail, $filterRow);
            }
        
            $response->addMessage(new Message($this->logEnvioAF, 'W'));
                
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
}