function SolicitacaoComprasUnidadeController(General, ScreenService, localStorageEngine, templateManager, MessageService) {

    var self = this;            

    this.validaProduto = function(prodInicial, prodFinal) {
        if (!General.comparaPratoProduto(prodInicial, prodFinal, getLocalVar('CDPICTPROD'))) {
            MessageService.showMessageByCode("PRODUTO_INICIAL_MENOR_IGUAL_FINAL");
            return false;
        }

        return true;
        
    };

    this.criaRelatorio = function(widget) {
        if (!widget.isValid()) return;
        
        if (!this.validaProduto(widget.currentRow.CDPRODINI, widget.currentRow.CDPRODFIN)) return;
        
        const reportName  = widget.currentRow.REPORT_NAME;
        
        General.openReportPopup(widget, self.abreRelatorio, [widget, widget.currentRow, reportName]);
    };
    
    this.setFilLanc = function(widget){
        var row = widget.currentRow;
        var reg = !Util.isEmptyOrBlank(row.REGIAO) ? row.REGIAO.join('_') : 'T';
        var dataSourceFilter = [
            { "name": "P_CDREGIAO", "operator": "=", "value": reg },
            { "name": "NRORG", "operator": "=", "value": getLocalVar('NRORG') },
            { "name": "CDOPERADOR", "operator": "=", "value": getLocalVar('CDOPERADOR') },
            { "operator": "ORDER_BY", "name": [ { "name": "CDFILIAL", "direction": "ASC" } ] }
        ];
        widget.getField('CDFILLANC').dataSourceFilter = dataSourceFilter;
    };
    
    this.setSol = function(widget){
        var row = widget.currentRow;
        var lanc = !Util.isEmptyOrBlank(row.CDFILLANC) ? row.CDFILLANC.join('_') : 'T';
        var filSoli = !Util.isEmptyOrBlank(row.CDFILSOLI) ? row.CDFILSOLI.join('_') : 'T';
        var dataSourceFilter = [
            { "name": "P_CDFILANCA", "operator": "=", "value": lanc },
            { "name": "P_TCDFILANCA", "operator": "=", "value": lanc },
            { "name": "P_CDFILSOLI", "operator": "=", "value": filSoli },
            { "name": "P_TCDFILSOLI", "operator": "=", "value": filSoli },
            { "name": "NRORG", "operator": "=", "value": getLocalVar('NRORG') }
        ];
        widget.getField('SOL').dataSourceFilter = dataSourceFilter;
    };
    
    this.setInitValues = function(widget) {
        var valIdioma = 'B';
        
        if(getLocalVar('language') == 'pt_br') {
            valIdioma = 'B';
        } else if(getLocalVar('language') == 'es_cl') {
            valIdioma = 'E';
        } else if(getLocalVar('language') == 'en_us') {
            valIdioma = 'I';
        }
        
        var dataSourceFilter = [
            { "name": "P_STIDIOMA", "operator": "=", "value": valIdioma }
        ];
        
        widget.currentRow.INTERVALO_DATA = {
            'formattedStart': moment().format('L'), 
            'formattedEnd': moment().format('L'), 
            'start': moment().format('L'), 
            'end': moment().format('L')
        };
        widget.currentRow.CDFILSOLI = [ getLocalVar("CDFILIAL") ];
        widget.currentRow.NMFILSOLI = [ getLocalVar("NMFILIAL") ];
        
        const filialField = widget.getField("CDFILSOLI");
        filialField.reload();
        
        General.callRestEngine("cmp#/getCdFilCentCmp", "Row", {}).then(function(response) {
            if(response.dataset.data.CDFILCENTCMP == response.dataset.data.CDFILIAL) {
                var NMFILCENTCMP = response.dataset.data.NMFILCENTCMP;
                var CDFILCENTCMP = response.dataset.data.CDFILCENTCMP;
                widget.currentRow.CDFILSOLI = [ CDFILCENTCMP ];
                widget.currentRow.NMFILSOLI = [ NMFILCENTCMP ];
                filialField.readOnly = false;
                filialField.reload().then(function(success) {
                    let verificador = 0;
                    success.dataset.filial_filioper_regiao.forEach(function(filial) {
                        if(filial.CDFILIAL == widget.currentRow.CDFILSOLI[0]) {
                            verificador++;
                        }
                    });
                    if(verificador == 0) {
                        delete widget.currentRow.CDFILSOLI;
                        delete widget.currentRow.NMFILSOLI;
                        filialField.reload();
                    }
                });
            } else {
                filialField.readOnly = true;
                filialField.reload().then(function(success) {
                    if(filialField.readOnly == true) {
                        let verificador = 0;
                        success.dataset.filial_filioper_regiao.forEach(function(filial) {
                            if(filial.CDFILIAL == widget.currentRow.CDFILSOLI[0]) {
                                verificador++;
                            }
                        });
                        if(verificador == 0) {
                            filialField.validations.required = true;
    	                    MessageService.showMessageByCode("FILIAL_SEM_REGIAO", null, { type: 'ERROR' });
    	                    delete widget.currentRow.CDFILSOLI;
    	                    delete widget.currentRow.NMFILSOLI;
    	                    filialField.reload();
                        } else {
                            delete filialField.validations.required;
    	                    filialField.reload();
    	                }
                    }
                });
            }
        }, function(error) {
            MessageService.showMessageByCode(error.data ? 'cmp#' + error.data.error : 'Timeout', null, { type: 'ERROR' });
            
            widget.currentRow.CDFILSOLI = [ getLocalVar("CDFILIAL") ];
            widget.currentRow.NMFILSOLI = [ getLocalVar("NMFILIAL") ];
            
            filialField.reload();
        });
        
        General.callRestEngine("cmp_solicitacao_solicitacaoCompras#/cmp03100_tipo_custo", "Row", {}).then(function(response) {
            if(response.dataset.dataset.length == 0) {
                General.callRestEngine('cmp_solicitacao_relatorios#/filtraCusto', 'FilterData', dataSourceFilter).then(function(callback) {
                    widget.getField('NMTIPOCUSTO').dataSource.data = callback.dataset.custo;
                    
                    widget.currentRow.CDTIPOCUSTO = widget.getField('NMTIPOCUSTO').dataSource.data[0].CDTIPOCUSTO;
                    widget.currentRow.NMTIPOCUSTO = widget.getField('NMTIPOCUSTO').dataSource.data[0].NMTIPOCUSTO;
                    
                    widget.redraw();
                }, function(err) {
        			ScreenService.showMessage(err.data.error);
        		});
            } else {
                General.callRestEngine('cmp_solicitacao_relatorios#/filtraCusto', 'FilterData', dataSourceFilter).then(function(callback) {
                    widget.getField('NMTIPOCUSTO').dataSource.data = callback.dataset.custo;
                    
                    widget.currentRow.CDTIPOCUSTO = response.dataset.dataset[0].CDTPCUSTCMP;
                    widget.currentRow.NMTIPOCUSTO = response.dataset.dataset[0].NMTIPOCUSTO;
                    
                    callback.dataset.custo.forEach(cd => {
                        if(cd.CDTIPOCUSTO == response.dataset.dataset[0].CDTPCUSTCMP) {
                            widget.currentRow.CDTIPOCUSTO = cd.CDTIPOCUSTO;
                            widget.currentRow.NMTIPOCUSTO = cd.NMTIPOCUSTO;
                        }
                    });
                    
                    widget.redraw();
                }, function(err) {
        			ScreenService.showMessage(err.data.error);
        		});
            }
            
            widget.getField('NMTIPOCUSTO').reload();
        }).catch(function(error) {
            ScreenService.showMessage(error.data.error);
        });
        
        widget.redraw();
    };
     
    
    this.abreRelatorio = function(widget, row, reportName) {
        row = angular.copy(widget.currentRow);

        row.CDFILSOLI = row.CDFILSOLI ? Array.isArray(row.CDFILSOLI) && row.CDFILSOLI.length > 0 ? row.CDFILSOLI.join('_') : 'T' : 'T';
        row.CDFILLANC = row.CDFILLANC ? Array.isArray(row.CDFILLANC) && row.CDFILLANC.length > 0 ? row.CDFILLANC.join('_') : 'T' : 'T';
        row.REGIAO    = !Util.isEmptyOrBlank(row.REGIAO) ? row.REGIAO.join('_') : 'T';
        row.GROUPFIL  = row.CDGRUPFILI ? row.CDGRUPFILI : 'T';
        row.SOL       = !Util.isEmptyOrBlank(row.SOL) ? row.SOL.join('_') : 'T';
        row.DTINI     = row.DTINI = row.INTERVALO_DATA.formattedStart;
        row.DTFIM     = row.DTFIM = row.INTERVALO_DATA.formattedEnd;
        
        row.CDFILSOLI1 = row.CDFILSOLI;
        row.REGIAO1    = row.REGIAO;
        row.GROUPFIL1  = row.GROUPFIL;
        row.CDFILLANC1 = row.CDFILLANC;
        row.SOL1       = row.SOL;
        row.DTINI1     = row.DTINI;
        row.DTFIM1     = row.DTFIM;
        
        if(row.CDPRODFIN){
            row.PRODFIN = row.CDPRODFIN.padEnd(20,'Z');
            row.PRODINI = row.CDPRODINI;
        } else {
            row.PRODFIN = "";
            row.PRODINI = "";
            row.CDPRODFIN = "";
        }
   	    General.openReport('cmp_solicitacao_relatorios#/solicitacaoComprasUnidade', reportName, row);   
    };
}
Configuration(function(ContextRegister) {
    ContextRegister.register('SolicitacaoComprasUnidadeController', SolicitacaoComprasUnidadeController);
});